unit shutdown;
{
  Name          TShutdown
  Description   TShutdown either logs off, shuts down, powers off or
                restarts the system (Windows 95/98/ME/NT/2K/XP).
  Version       0.92 10/10/02
  Author        Dennis Spreen (dennis@spreendigital.de)
  Code          32-Bit Delphi 2+
  License       Freeware
  DSPDate       06/16/99 ;) After a long field test, the component is now
                released to the DSP public ;)
  History       05/01/97    component icon changed
                04/29/97    added Windows 95 Message Box (like WinNT)
                04/28/97    added Windows NT functions
                05/01/97    v0.91 - Initial public release  
                10/10/02    v0.92 - Updated auto os detection and removed message box support
  Homepage      http://www.spreendigital.de/delphi
}

interface

uses
  Windows, SysUtils, Classes;

type TShOs=(shAuto,shWin32s,shWin95,shWin98,shWinME,shWinNT,shWin2K,shWinXP,shWinUnknown);
     TShutdownOption=(shoForce,shoLogoff,shoPoweroff,shoReboot,shoShutdown);
     TshutdownOptions=set of TShutDownOption;

type
  TShutdown = class(TComponent)
  private
  protected
    FOS: TShOs;
    FOptions: TShutdownOptions;
    FShutOptions: Integer;
    function GetOSVersion:TShOs;
    function GetOperatingSystem: TShOs;
    function ShutDown95: Boolean;
    function ShutDownNT: Boolean;
    property ShutOptions: Integer read FShutOptions write FShutOptions;
  public
  published
    property OperatingSystem: TShOS read GetOperatingSystem write FOS default shAuto;
    property Options: TShutDownOptions read FOptions write FOptions default [];
    function Execute:boolean;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('FreeWare', [TShutdown]);
end;


function TShutdown.GetOSVersion: TShOs;
var OSversion: OSVERSIONINFO;
begin
  OSversion.dwOSVersionInfoSize := sizeof(OSVERSIONINFO);
  GetVersionEx(OSversion);
  result := shWinUnknown;
  case (OSversion.dwPlatformId) of
   VER_PLATFORM_WIN32s: Result := shWin32s;
   VER_PLATFORM_WIN32_WINDOWS:
     begin
       if(OSversion.dwMinorVersion=0) then
         result := shWin95
       else
       if(OSversion.dwMinorVersion=10) then
      	 result := shWin98
       else
       if(OSversion.dwMinorVersion=90) then
 	 result := shWinME;
     end;
   VER_PLATFORM_WIN32_NT:
     begin
       if(OSversion.dwMajorVersion=5) and (OSversion.dwMinorVersion=0) then
         result := shWin2K
       else
       if(OSversion.dwMajorVersion=5) and (OSversion.dwMinorVersion=1) then
         result := shWinXP
       else
       if(OSversion.dwMajorVersion<=4) then
         result := shWinNT;
     end
   end;
end;


function TShutDown.GetOperatingSystem: TShOs;
begin
 if FOS = shAuto then FOS := GetOSVersion;
 result := FOS;
end;


function TShutDown.Execute: Boolean;
begin
 if (OperatingSystem = shWinNT) or
    (OperatingSystem = shWin2K) or
    (OperatingSystem = shWinXP) or
    (OperatingSystem = shWinUnknown) then
   result := ShutDownNT
 else
   result := ShutDown95;
end;


function TShutDown.ShutDownNT: Boolean;
const
  ADJUST_PRIV = TOKEN_QUERY or TOKEN_ADJUST_PRIVILEGES;
  SHTDWN_PRIV = 'SeShutdownPrivilege';
  PRIV_SIZE   = sizeOf(TTokenPrivileges);
var
  Len: DWORD;
  TokenPriv, Dummy: TTokenPrivileges;
  Token: THandle;
  Error: integer;

begin
 error := 0;
  // Get a token for this process.

  // Delphi2:
  //if not OpenProcessToken(GetCurrentProcess(), ADJUST_PRIV, @Token) then
  if not OpenProcessToken(GetCurrentProcess(), ADJUST_PRIV, Token) then
      Error:=Error or 4;
    {    ShowMessage('OpenProcessToken failed');}

  // Get the LUID for the shutdown privilege.
  if not LookupPrivilegeValue(nil, SHTDWN_PRIV,TokenPriv.Privileges[0].Luid) then
    Error:=Error or 8;
    {ShowMessage('LookupPrivilegeValue failed');}

  // Adjust shutdown privilege for this process.
  TokenPriv.Privileges[0].Attributes := SE_PRIVILEGE_ENABLED;
  TokenPriv.PrivilegeCount := 1;  // One privilege to set
  if not AdjustTokenPrivileges(Token, false, TokenPriv, PRIV_SIZE,Dummy, Len) then
    Error:=Error or 16;
   {ShowMessage('AdjustTokenPrivileges failed');}

   if (Error=0) then
     Result := ShutDown95
   else Result := false;
end;



function TShutDown.ShutDown95:boolean;
begin
 Shutoptions := 0;

 if (shoForce in Options)    then ShutOptions := ShutOptions or EWX_Force;
 if (shoLogOff in Options)   then ShutOptions := ShutOptions or EWX_LogOff;
 if (shoPoweroff in Options) then ShutOptions := ShutOptions or EWX_PowerOff;
 if (shoReboot in Options)   then ShutOptions := ShutOptions or EWX_Reboot;
 if (shoShutdown in Options) then ShutOptions := ShutOptions or EWX_ShutDown;

 exitWindowsEx(ShutOptions,0);
 result := true;
end;



end.
